package edu.lawrence.summary;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

public class SummaryDAO {
    public SummaryDAO() {
        // Load the database driver
        try {
            Class.forName("com.mysql.cj.jdbc.Driver");
        } catch (ClassNotFoundException ex) {
            System.out.println("Could not load database driver.");
        }
        // Open the connection and set up statements
        try {
            connection = DriverManager.getConnection("jdbc:mysql://localhost:3306/quiz?user=student&password=Cmsc250!");
            statement = connection.createStatement();
            idFromNameStmt = connection.prepareStatement(idFromNameSQL);
            idFromQuizStmt = connection.prepareStatement(idFromQuizSQL);
            studentSummaryStmt = connection.prepareStatement(studentSummarySQL);
            quizSummaryStmt = connection.prepareStatement(quizSummarySQL);
        } catch (SQLException ex) {
            System.out.println("Could not connect to database.");
            ex.printStackTrace();
        }

    }
    
    public ArrayList<String> getStudentNames() {
        ArrayList<String> results = new ArrayList<String>();
        try {
            ResultSet resultSet = statement.executeQuery("select name from students");

            while (resultSet.next()) {
                results.add(resultSet.getString(1));
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        return results;    
    }
    
    public ArrayList<String> getQuizNames() {
        ArrayList<String> results = new ArrayList<String>();
        try {
            ResultSet resultSet = statement.executeQuery("select title from quizzes");

            while (resultSet.next()) {
                results.add(resultSet.getString(1));
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        return results;    
    }
    
    public String generateSummaryForStudent(String name) {
        int id = 0;
        String result = "";
        try {
            idFromNameStmt.setString(1, name);
            ResultSet resultSet = idFromNameStmt.executeQuery();
            if(resultSet.next()) {
                id = resultSet.getInt("id");
            } else
                return "Invalid student name";
            studentSummaryStmt.setInt(1, id);
            resultSet = studentSummaryStmt.executeQuery();
            result = "quiz correct answered\n";
            while(resultSet.next()) {
                String line = "\n" + resultSet.getInt("quiz") + ": " + resultSet.getInt("correct") + "   " + resultSet.getInt("answered");
                result = result + line;
            }
        } catch(SQLException ex) {
            ex.printStackTrace();
        }
        return result;        
    }
    
    public String generateSummaryForQuiz(String quiz) {
        int id = 0;
        String result = "";
        try {
            idFromQuizStmt.setString(1, quiz);
            ResultSet resultSet = idFromQuizStmt.executeQuery();
            if(resultSet.next()) {
                id = resultSet.getInt("id");
            } else
                return "Invalid quiz name";
            quizSummaryStmt.setInt(1, id);
            resultSet = quizSummaryStmt.executeQuery();
            result = "name         correct answered\n";
            while(resultSet.next()) {
                String line = "\n" + resultSet.getString("name") + ": " + resultSet.getInt("correct") + "   " + resultSet.getInt("answered");
                result = result + line;
            }
        } catch(SQLException ex) {
            ex.printStackTrace();
        }
        return result;        
    }
    
    private Connection connection;
    private Statement statement;
    private String idFromNameSQL = "select id from students where name=?";
    private PreparedStatement idFromNameStmt;
    private String studentSummarySQL = "select quiz, sum(correct) as correct, count(question) as answered from results where student=? group by quiz";
    private PreparedStatement studentSummaryStmt;
    private String idFromQuizSQL = "select id from quizzes where title=?";
    private PreparedStatement idFromQuizStmt;
    private String quizSummarySQL = "select name, sum(correct) as correct, count(question) as answered from results join students on results.student=students.id where quiz=? group by student";
    private PreparedStatement quizSummaryStmt;    
}
